@@ -3,7 +3,7 @@ package Shipwright::Backend::FS;
 use warnings;
 use strict;
 use Carp;
-use File::Spec::Functions qw/catfile catdir/;
+use File::Spec::Functions qw/catfile splitdir/;
 use Shipwright::Util;
 use File::Copy qw/copy/;
 use File::Copy::Recursive qw/dircopy/;
@@ -51,52 +51,83 @@ sub _cmd {
         croak "$type need option $_" unless $args{$_};
     }
 
-    my $cmd;
+    my @cmd;
 
     if ( $type eq 'checkout' || $type eq 'export' ) {
-        $cmd = [ 'cp', '-r', $self->repository . $args{path}, $args{target} ];
+        @cmd = [ 'cp', '-r', $self->repository . $args{path}, $args{target} ];
     }
     elsif ( $type eq 'import' ) {
         if ( $args{_extra_tests} ) {
-            $cmd =
+            @cmd =
               [ 'cp', '-r', $args{source}, $self->repository . '/t/extra' ];
         }
         else {
             if ( my $script_dir = $args{build_script} ) {
-                $cmd = [
+                push @cmd,
+                  [
                     'cp', '-r', "$script_dir/",
                     $self->repository . "/scripts/$args{name}",
-                ];
+                  ];
             }
             else {
-                $cmd = [
-                    'cp', '-r', "$args{source}/",
-                    $self->repository . "/dists/$args{name}",
-                ];
+                if ( $self->has_branch_support ) {
+                    my @dirs = splitdir( $args{as} );
+                    unless (
+                          -e $self->repository
+                        . "/sources/$args{name}/"
+                        . join '/',
+                        @dirs[ 0 .. $#dirs - 1 ]
+                      )
+                    {
+                        push @cmd,
+                          [
+                            'mkdir',
+                            '-p',
+                            $self->repository
+                              . "/sources/$args{name}/"
+                              . join '/',
+                            @dirs[ 0 .. $#dirs - 1 ]
+                          ];
+                    }
+
+                    push @cmd,
+                      [
+                        'cp', '-r', "$args{source}/",
+                        $self->repository . "/sources/$args{name}/$args{as}",
+                      ];
+                }
+                else {
+                    push @cmd,
+                      [
+                        'cp', '-r', "$args{source}/",
+                        $self->repository . "/dists/$args{name}",
+                      ];
+
+                }
             }
         }
     }
     elsif ( $type eq 'delete' ) {
-        $cmd = [ 'rm', '-rf', $self->repository . $args{path}, ];
+        @cmd = [ 'rm', '-rf', $self->repository . $args{path}, ];
     }
     elsif ( $type eq 'move' ) {
-        $cmd = [
+        @cmd = [
             'mv',
             $self->repository . $args{path},
             $self->repository . $args{new_path}
         ];
     }
     elsif ( $type eq 'info' || $type eq 'list' ) {
-        $cmd = [ 'ls', $self->repository . $args{path} ];
+        @cmd = [ 'ls', $self->repository . $args{path} ];
     }
     elsif ( $type eq 'cat' ) {
-        $cmd = [ 'cat', $self->repository . $args{path} ];
+        @cmd = [ 'cat', $self->repository . $args{path} ];
     }
     else {
         croak "invalid command: $type";
     }
 
-    return $cmd;
+    return @cmd;
 }
 
 =item _yml
@@ -176,3 +207,16 @@ sub import {
 =cut
 
 1;
+
+__END__
+
+=head1 AUTHORS
+
+sunnavy  C<< <sunnavy@bestpractical.com> >>
+
+=head1 LICENCE AND COPYRIGHT
+
+Shipwright is Copyright 2007-2009 Best Practical Solutions, LLC.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
